package org.act.services.connector;
import java.util.ArrayList;
import java.util.List;
import org.act.metaset.Metaset;
import org.act.services.ConfigException;
import org.act.services.IMessageContext;
import org.act.services.ServicesObject;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.ServicesProtocol;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.ObjectProducerThreadPoolRunnable;
import org.act.services.thread.Terminatable;

public class ServicesObjectConnector extends AObjectPoolConnector {
	
	public ServicesObjectConnector(IRuntime runtime) {
		super(runtime);
	}
	private IObjectPool msgIn;
	private IObjectPool msgOut;
	private IObjectPool soIn;
	private IObjectPool soOut;
	private Terminatable msg2so;
	private Terminatable so2msg;
	private List messages = new ArrayList();
	
	public Object adapt(Object obj) {
		return null;
	}
	public void config(Metaset conf) throws ConfigException {
		super.config(conf);
		String msg = conf.selector(".\\", "inputpool");
		String so = conf.selector(".\\", "outputpool");
		msgIn =
			runtime.getObjectPool(msg, IObjectPool.IN_MODE);
		msgOut =
			runtime.getObjectPool(msg, IObjectPool.OUT_MODE);
		soIn =
			runtime.getObjectPool(so, IObjectPool.IN_MODE);
		soOut =
			runtime.getObjectPool(so, IObjectPool.OUT_MODE);
		
				Metaset protocols = conf.filter(".\\protocol");
		for (int i=0; i<protocols.getChilds().size(); i++) {
			try {
				Metaset protocol = (Metaset) protocols.getChilds().get(i);
				String classname = protocol.selector(".\\","class");
				Class cls = Class.forName(classname);
				if (ServicesProtocol.class.isAssignableFrom(cls)) {
					ServicesProtocol servicesProtocol = (ServicesProtocol) cls.newInstance();
					servicesProtocol.config(protocol);
					messages.add(servicesProtocol);
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ConfigException e) {
				e.printStackTrace();
			}
		}
		
		msg2so = new Terminatable() {
			
			protected void process() throws ObjectOperationException {
								IMessageContext request = null;
				synchronized (msgIn) {
					while (msgIn.empty()) {
						switch (getState()) {
						case RUNNING:
							try {
								msgIn.wait();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;
						case STOPPING:
							return;
						default:
							throw new IllegalMonitorStateException();
						}
					}
					request = (IMessageContext) msgIn.get();
					msgIn.notifyAll();
			}
							for (int i=0; i<messages.size(); i++) {
				ServicesProtocol comsumer = (ServicesProtocol) messages.get(i);
				Object adaptee = comsumer.adapt(request);
				if (adaptee!=null && adaptee instanceof ObjectProducerThreadPoolRunnable) {
					ObjectProducerThreadPoolRunnable runnable = (ObjectProducerThreadPoolRunnable)adaptee;
					runnable.setReceiver(soIn);
					runnable.runIt(null);
										break;
				}
			}
			}
		};
		
		so2msg = new Terminatable() {
			
			protected void process() throws ObjectOperationException {
								ServicesObject request = null;
				synchronized (soOut) {
					while (soOut.empty()) {
						switch (getState()) {
						case RUNNING:
							try {
								soOut.wait();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;
						case STOPPING:
							return;
						default:
							throw new IllegalMonitorStateException();
						}
					}
					request = (ServicesObject) soOut.get();
					soOut.notifyAll();
				}
								for (int i=0; i<messages.size(); i++) {
					ServicesProtocol comsumer = (ServicesProtocol) messages.get(i);
					Object adaptee = comsumer.adapt(request);
					if (adaptee!=null && adaptee instanceof ObjectProducerThreadPoolRunnable) {
						ObjectProducerThreadPoolRunnable runnable = (ObjectProducerThreadPoolRunnable)adaptee;
						runnable.setReceiver(msgOut);
						runnable.runIt(null);
												break;
					}
				}
			}
		};
	}
	public void stop() {
		msg2so.terminate();
		synchronized (msgIn) {
			msgIn.notifyAll();
		}
		so2msg.terminate();
		synchronized (soOut) {
			soOut.notifyAll();
		}
	}
	public void start() {
		Thread t = new Thread(msg2so);		t.setName("WSDL Input");
		t.start();
		Thread t2 = new Thread(so2msg);		t2.setName("WSDL Output");
		t2.start();
	}
}
