
package org.act.services.transaction.manager;


import java.lang.reflect.Method;
import java.util.Vector;
import org.act.services.transaction.exception.TxException;
import org.act.services.transaction.information.DBLog;
import org.act.services.transaction.messages.Address;
import org.act.services.transaction.messages.ApplicationMessage;
import org.act.services.transaction.messages.Begin;
import org.act.services.transaction.messages.Begun;
import org.act.services.transaction.messages.Cancel;
import org.act.services.transaction.messages.Cancelled;
import org.act.services.transaction.messages.Confirm;
import org.act.services.transaction.messages.ConfirmOnePhase;
import org.act.services.transaction.messages.Confirmed;
import org.act.services.transaction.messages.Context;
import org.act.services.transaction.messages.ContextReply;
import org.act.services.transaction.messages.Enrol;
import org.act.services.transaction.messages.Fault;
import org.act.services.transaction.messages.Hazard;
import org.act.services.transaction.messages.InferiorState;
import org.act.services.transaction.messages.Message;
import org.act.services.transaction.messages.Prepare;
import org.act.services.transaction.messages.PrepareInferiors;
import org.act.services.transaction.messages.Prepared;
import org.act.services.transaction.messages.Redirect;
import org.act.services.transaction.messages.RelatedGroup;
import org.act.services.transaction.messages.RequestInferiorStatuses;
import org.act.services.transaction.messages.Resign;
import org.act.services.transaction.messages.Status;
import org.act.services.transaction.messages.SuperiorState;
import org.act.services.transaction.participant.InferiorContext;
import org.act.services.transaction.participant.Vote;
import org.act.services.transaction.util.LogCategory;
import org.act.services.transaction.util.MessageUtil;
import org.act.services.transaction.util.Timer;
import org.act.services.transaction.util.TxUri;
import org.act.services.transaction.util.XElement;
import org.apache.log4j.Category;
public class SubDecider extends Superior{
	
    protected InferiorContext ic = null;
    
    boolean isContextBegin = true;
    
    public SubDecider()
    {
        super(LogCategory.getCategory("Э"));
    }
    
    protected SubDecider(Category logger)
    {
        super(logger);
    }
    
    public String onMessage(String xml)
    {
        Message retMsg = null;
        try
        {
            Message msg = MessageUtil.parseMessage(xml);
            switch (msg.getType())
            {
                case Message.TYPE_RELATED_GROUP:
                    RelatedGroup rg = (RelatedGroup)msg;
                    retMsg = onRelatedGroup(rg);
                    break;
                case Message.TYPE_PREPARE:
                    retMsg = onPrepare((Prepare)msg);
                    break;
                case Message.TYPE_CONFIRM:
                    retMsg = onConfirm((Confirm)msg);
                    break;
                case Message.TYPE_CONFIRMED:
                    retMsg = onConfirmed((Confirmed)msg);
                    break;
                case Message.TYPE_CANCEL:
                    retMsg = onCancel((Cancel)msg);
                    break;
                case Message.TYPE_CANCELLED:
                    retMsg = onCancelled((Cancelled)msg);
                    break;
                case Message.TYPE_CONFIRM_ONE_PHASE:
                    retMsg = onConfirmOnePhase((ConfirmOnePhase)msg);
                    break;
                case Message.TYPE_CONTEXT_REPLY:
                    retMsg = onContextReply((ContextReply)msg);
                    break;
                case Message.TYPE_ENROL:
                    retMsg = onEnrol((Enrol)msg);
                    break;
                case Message.TYPE_FAULT:
                    retMsg = onFault((Fault)msg);
                    break;
                case Message.TYPE_HAZARD:
                    retMsg = onHazard((Hazard)msg);
                    break;
                case Message.TYPE_PREPARE_INFERIORS:
                    retMsg = onPrepareInferiors((PrepareInferiors)msg);
                    break;
                case Message.TYPE_REDIRECT:
                    retMsg = onRedirect((Redirect)msg);
                    break;
                case Message.TYPE_RESIGN:
                    retMsg = onResign((Resign)msg);
                    break;
                case Message.TYPE_SUPERIOR_STATE:
                    retMsg = onSuperiorState((SuperiorState)msg);
                    break;
                case Message.TYPE_STATUS:
                    retMsg = onStatus((Status)msg);
                    break;
                case Message.TYPE_REQUEST_INFERIOR_STATUSES:
                    retMsg = onRequestInferiorStatuses((RequestInferiorStatuses)
                        msg);
                    break;
                                    case Message.TYPE_BEGIN:
                case Message.TYPE_ENROLLED:
                case Message.TYPE_PREPARED:
                case Message.TYPE_ACT_ADMIN:
                case Message.TYPE_CANCEL_TRANSACTION:
                case Message.TYPE_CONFIRM_TRANSACTION:
                case Message.TYPE_CONTRADICTION:
                    retMsg = onError(msg);
                    break;
                                    case Message.TYPE_UNKNOWN:
                default:
                    retMsg = onApplicationMessage((ApplicationMessage)msg);
            }
        }
        catch (Exception e)
        {
                        logger().debug("(Э) onMessage Error:" + e.getMessage());
            retMsg = new Fault(null, null,
                               "SubDecider onMessage Exception: " +
                               e.getMessage());
        }
        return retMsg.toString();
    }
    
    public Message onError(Message msg)
    {
        return reportFault("subDecider should not receive such a message:"
                           + msg.getName());
    }
    Message onRelatedGroup(RelatedGroup msg)
    {
        if (msg.getSubtype() == RelatedGroup.RG_BEGIN_CONTEXT)
        {
            isContextBegin = true;
            logger().debug("(Э)յĵBeginϢ");
            return onRelatedGroup1(msg);
        }
        if (msg.getSubtype() == RelatedGroup.RG_CONTEXT_APP)
        {
            isContextBegin = false;
            logger().debug("(Э)յĵapplicationϢ");
            return onRelatedGroup2(msg);
        }
        return reportFault("subDecider received wrong message: " + msg.toString());
    }
    
    Message onRelatedGroup1(RelatedGroup beginContext)
    {
                Begin begin = beginContext.getBegin();
        Context parentContext = beginContext.getContext();
        boolean atom = begin.getAtom();
                txId = parentContext.getTransactionId();
        Begun begun = new Begun(txId);
                Message subContext = onContext(parentContext);
        if (subContext instanceof Fault)
        {
            return subContext;
        }
        if (!atom)
        {
            ((Context)subContext).setSuperiorType("cohesion");
        }
        RelatedGroup rg = new RelatedGroup(RelatedGroup.RG_BEGUN_CONTEXT);
        rg.addMessage(subContext);
        rg.addMessage(begun);
        control.setTxStatus(TxStatus.TX_BEGUN);
        rg.setTransactionId(txId);
        return rg;
    }
    
    public Message onRelatedGroup2(RelatedGroup msg)
    {
        RelatedGroup ret = new RelatedGroup();
        Context context = msg.getContext();
                Message contextReply = onContext(context);
        if (contextReply.getType() == Message.TYPE_FAULT)
        {
            return contextReply;
        }
        ret.addMessage(contextReply);
                ApplicationMessage appMsg = (ApplicationMessage)
                                    msg.getMessageByType(Message.TYPE_UNKNOWN);
        Message appResponse = null;
        if (appMsg != null)
        {
                        appResponse = onApplicationMessage(appMsg);
        }
        ret.addMessage(appResponse);
        ret.setTransactionId(txId);
        return ret;
    }
    
    public Message onContext(Context context)
    {
        Message response = null;
        if (context == null)
        {
            logger().debug("ЭܵϢҪϢ");
            return reportFault(
                "context is not in the message which the subDecider receives");
        }
        String superiorId = context.getSuperiorIdentifier();
        if (txId == null)
        {
            txId = context.getTransactionId();
        }
        String subDiciderAddr = this.getWebServiceUrl();
        String subDiciderId = TxUri.buildUri(subDiciderAddr, txId);
        logger().debug("[" + Timer.getCNTime() + "]"
                       + "[" + subDiciderId + "]ʼ");
        SuperiorContext sc = control.getSuperiorContext();
                if (sc == null)
        {
            sc = new SuperiorContext(txId);
            sc.setTopTransaction(false);
            sc.setSuperiorId(subDiciderId);
            sc.setSuperiorAddress(subDiciderAddr);
            control.setSuperiorContext(sc);
                        DBLog.logSuperior(sc);
                        control.setTxStatus(TxStatus.TX_BEGINNING);
        }
                if (ic == null)
        {
            boolean enrolled = false;
                        Enrol enrol = new Enrol();
            enrol.setSuperiorIdentifier(superiorId);
            enrol.setInferiorIdentifier(subDiciderId);
            Address add = new Address(subDiciderAddr);
            enrol.addInferiorAddress(add);
                        enrol.setResponseRequested(true);
            enrol.setTransactionId(txId);
                        Vector supAdds = context.getSuperiorAddresses();
            String superiorAddrress = null;
            try
            {
                for (int i = 0; i < supAdds.size(); i++)
                {
                    superiorAddrress = ((Address)supAdds.get(i)).getAddress();
                    Message reply=new Message("fsdf",2);
                                              if (reply.getType() == Message.TYPE_ENROLLED)
                    {
                        enrolled = true;
                    }
                }
            }
            catch (Exception e)
            {
                logger().error("()[" +
                               TxUri.getUrlFromUri(subDiciderId) +
                               "]Ǽʧ:" +
                               e.getMessage());
            }
            if (enrolled == true)
            {
                Resource r = this;
                ic = new InferiorContext(txId);
                                ic.setSuperiorId(superiorId);
                ic.setSuperiorAddress(superiorAddrress);
                                ic.setInferiorId(subDiciderId);
                ic.setInfeirorAddress(subDiciderAddr);
                ic.setStartTime(System.currentTimeMillis());
                                ic.setStatus(Status.STATUS_ACTIVE);
                                r.enrolled(txId);
            }
            else
            {
                return reportFault("subDiceder[" + subDiciderId +
                                   "] cannot be enrolled");
            }
        }
        if (isContextBegin)
        {
            Context subContext = new Context(subDiciderAddr, subDiciderId);
            return subContext;
        }
        else
        {
            ContextReply cxReply = new ContextReply(superiorId,
                ContextReply.
                ContextReply_completed);
            cxReply.setInferiorId(subDiciderId);
            logger().debug("()[" + subDiciderId +
                           "]ѾǼǣContextreplyϢ");
            return cxReply;
        }
    }
    
    protected Message onApplicationMessage(ApplicationMessage appMsg)
    {
        Message appResponse = null;
        String opName = appMsg.getOperationName();
        XElement paramEle = appMsg.getParamElement();
                Class appClass = this.getClass();
        String error = null;
        try
        {
            Method method = appClass.getMethod(opName, new Class[]
                                               {XElement.class});
            XElement result = (XElement)method.invoke(this,
                new Object[]
                {paramEle});
            XElement resEle = new XElement(opName + "Response");
            resEle.addElement(result);
            appResponse = new ApplicationMessage(resEle);
        }
        catch (java.lang.NoSuchMethodException nme)
        {
            error = "No such method : " + nme.getMessage();
        }
        catch (java.lang.IllegalAccessException le)
        {
            error = "the underlying method is inaccessible:" + le.getMessage();
        }
        catch (java.lang.IllegalArgumentException iae)
        {
            error = "the specified object argument is illegal: " +
                    iae.getMessage();
        }
        catch (java.lang.reflect.InvocationTargetException ite)
        {
            error = "the underlying method throws an exception :" +
                    ite.getMessage();
        }
        catch (Exception e)
        {
            error = e.getMessage();
        }
        finally
        {
                        if (error != null)
            {
                logger().error("÷" + opName + "ʱ쳣" + error);
                ic.setStatus(Status.STATUS_FAULT);
                return reportFault(opName + " error:" + error);
            }
            return appResponse;
        }
    }
    
    public Message onPrepare(Prepare prepare)
    {
        Message retMsg = null;
        String error = null;
        String infId = prepare.getInferiorIdentifier();
                String supId = null;
        logger().debug("SubDicider[" + infId + "]ʼpreprae");
        int result = Vote.VOTE_FAULT;
        SuperiorContext sc = control.getSuperiorContext();
        try
        {
            supId = ic.getSuperiorId();
                        if (sc.isAtom())
            {
                                result = control.prepare(txId);
                            }
            else
            {
                control.setTxStatus(TxStatus.TX_PREPARING);
                                result = prepare(txId);
                                control.setTxStatus(TxStatus.TX_PREPARED);
            }
        }
        catch (TxException te)
        {
            if (sc.isCohesion())
            {
                control.setTxStatus(TxStatus.TX_FAULT);
            }
            error = "SubDecider[" + infId +
                    "] cannot prepare:" + te.getMessage();
        }
                if (result == Vote.VOTE_PREPARED)
        {
            Prepared prepared = new Prepared();
            prepared.setDefaultIsCancel(true);
            prepared.setInferiorIdentifier(infId);
            prepared.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]prepraeͶƱ޳");
            prepared.setTransactionId(txId);
            retMsg = prepared;
        }
        else if (result == Vote.VOTE_ABORTED)
        {
            Cancelled cancelled = new Cancelled();
            cancelled.setInferiorIdentifier(infId);
            cancelled.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]prepraeͶƱ");
            cancelled.setTransactionId(txId);
            retMsg = cancelled;
        }
        else if (result == Vote.VOTE_RESIGN)
        {
            Resign resign = new Resign();
            resign.setInferiorIdentifier(infId);
            resign.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]prepraeͶƱ˳");
            resign.setTransactionId(txId);
            retMsg = resign;
        }
        else
        {
            error = "SubDecider[" + infId + "] cannot prepare";
        }
        if (error != null)
        {
            control.setTxStatus(TxStatus.TX_FAULT);
            logger().debug(error);
            return reportFault(error);
        }
        else
        {
            return retMsg;
        }
    }
    
    public Message onCancel(Cancel cancel)
    {
        String error = null;
        Message retMsg = null;
        String type = control.getTxType() ? "ԭ" : "";
        String infId = cancel.getInferiorIdentifier();
                String supId = null;
        int result = TxControl.TX_FAULT;
        logger().debug("SubDicider[" + infId + "]ʼcancel");
        SuperiorContext sc = control.getSuperiorContext();
        try
        {
            supId = ic.getSuperiorId();
            if (sc.isAtom())
            {
                result = control.cancel(txId);
            }
            else
            {
                control.setTxStatus(TxStatus.TX_CANCELLING);
                cancel(txId);
                result = TxControl.TX_CANCELLED;
                control.setTxStatus(TxStatus.TX_CANCELLED);
            }
        }
        catch (TxException te)
        {
            if (sc.isCohesion())
            {
                control.setTxStatus(TxStatus.TX_FAULT);
            }
            error = "SubDecider[" + infId + "] cannot cancel:"
                    + te.getMessage();
        }
        if (result == TxControl.TX_CANCELLED)
        {
            Cancelled cancelled = new Cancelled();
            cancelled.setInferiorIdentifier(infId);
            cancelled.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]cancel");
            cancelled.setTransactionId(txId);
            retMsg = cancelled;
        }
        else
        {
            error = "SubDecider[" + infId + "] can not cancel";
        }
        if (error != null)
        {
            control.setTxStatus(TxStatus.TX_FAULT);
            logger().debug(error);
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "ȡִ");
            return reportFault(error);
        }
        else
        {
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "ִ(ȡ)");
            return retMsg;
        }
    }
    
    public Message onConfirm(Confirm confirm)
    {
        String error = null;
        Message retMsg = null;
        String type = control.getTxType() ? "ԭ" : "";
        String infId = confirm.getInferiorIdentifier();
                String supId = null;
        logger().debug("SubDicider[" + infId + "]ʼconfirm");
        int result = TxControl.TX_FAULT;
        SuperiorContext sc = control.getSuperiorContext();
        try
        {
            supId = ic.getSuperiorId();
            if (sc.isAtom())
            {
                result = control.confirmOnly(txId);
            }
            else
            {
                control.setTxStatus(TxStatus.TX_CONFIRMING);
                confirm(txId);
                result = TxControl.TX_CONFIRMED;
                control.setTxStatus(TxStatus.TX_CONFIRMED);
            }
        }
        catch (TxException te)
        {
            if (sc.isCohesion())
            {
                control.setTxStatus(TxStatus.TX_FAULT);
            }
            error = "SubDecider[" + infId +
                    "] can not confirm:" + te.getMessage();
        }
        if (result == TxControl.TX_CONFIRMED)
        {
            Confirmed confirmed = new Confirmed();
            confirmed.setInferiorIdentifier(infId);
            confirmed.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]confirm");
            confirmed.setTransactionId(txId);
            retMsg = confirmed;
        }
        else
        {
            error = "SubDecider[" + infId +
                    "] cannot confirm";
        }
        if (error != null)
        {
            control.setTxStatus(TxStatus.TX_FAULT);
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "confirmִ:" + error);
            return reportFault(error);
        }
        else
        {
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "ύ");
            return retMsg;
        }
    }
    
    public Message onConfirmOnePhase(ConfirmOnePhase confirmonePhase)
    {
        String error = null;
        Message retMsg = null;
        String type = control.getTxType() ? "ԭ" : "";
        String infId = confirmonePhase.getInferiorIdentifier();
        String supId = null;
        logger().debug("SubDicider[" + infId + "]ʼconfirm_one_phase");
        int result = TxControl.TX_FAULT;
        SuperiorContext sc = control.getSuperiorContext();
        try
        {
            supId = ic.getSuperiorId();
            if (sc.isAtom())
            {
                                                                result = control.confirm(txId);
            }
            else
            {
                control.setTxStatus(TxStatus.TX_CONFIRMING);
                confirm_one_phase(txId);
                result = TxControl.TX_CONFIRMED;
                control.setTxStatus(TxStatus.TX_CONFIRMED);
            }
        }
        catch (TxException te)
        {
            if (sc.isCohesion())
            {
                control.setTxStatus(TxStatus.TX_FAULT);
            }
            error = "SubDecider[" + infId +
                    "] cannot confirm_one_phase:" + te.getMessage();
        }
        if (result == TxControl.TX_CONFIRMED)
        {
            Confirmed confirmed = new Confirmed();
            confirmed.setInferiorIdentifier(infId);
            confirmed.setSuperiorIdentifier(supId);
            logger().debug("SubDicider[" + infId + "]confirm_one_phase");
            confirmed.setTransactionId(txId);
            retMsg = confirmed;
        }
        else
        {
            error = "SubDecider[" + infId +
                    "] cannot confirm_one_phase";
        }
        if (error != null)
        {
            control.setTxStatus(TxStatus.TX_FAULT);
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "һ׶ύִ" + error);
            return reportFault(error);
        }
        else
        {
            logger().debug("[" + Timer.getCNTime() + "]" + type + "" + infId +
                           "һ׶ύ");
            return retMsg;
        }
    }
    
    public Message onSuperiorState(SuperiorState msg)
    {
        InferiorState is = new InferiorState(InferiorState.ACTIVE);
        is.setTransactionId(txId);
        return is;
    }
}
