
package org.act.services.transaction.messages;
import java.util.*;
import org.act.metaset.*;
import org.act.services.transaction.util.*;


public class Context extends Message{
	
    private Vector supaddr_ = new Vector();
    
    private String supid_;
    
    private boolean atom_ = true;
    
    private String txId = null;
  
    
    public Context(String superiorAddr, String supid)
    {
        super("btp:context", Message.TYPE_CONTEXT);
        supaddr_.addElement(new Address(superiorAddr));
        supid_ = supid;
    }
    
    public Context(XElement el)
    {
        super(el, Message.TYPE_CONTEXT);
        XHashtable h = el.toHashtable();
        String ns = el.getNamespace();
        ns = (ns == null ? "" : ns + ":");
        id_ = el.getAttribute("id");
        supid_ = pickString(h, "/" + name_ + "/" + ns + "superior-identifier");
        supaddr_ = pickAddrSet(h, "/" + name_ + "/" + ns + "superior-address");
        Object obj = h.getSingle("/" + name_ + "/" + ns + "superior-type");
        if (obj != null)
        {
            atom_ = !obj.toString().equalsIgnoreCase("cohesion");
        }
    }
    
    public String getSuperiorIdentifier()
    {
        return supid_;
    }
    
    public void setTransactionId(String txId)
    {
        this.txId = txId;
    }
    
    public String getTransactionId()
    {
        if (txId != null)
        {
            return txId;
        }
        else
        {
            return TxUri.getTransactionIdFromUri(supid_);
        }
    }
    
    public String getSuperiorUrl()
    {
        int pos = supid_.indexOf('#');
        return (pos <= 0 ? supid_ : supid_.substring(0, pos));
    }
    
    public void setSuperiorType(String atomOrcohesion)
    {
        atom_ = !atomOrcohesion.equalsIgnoreCase("cohesion");
    }
    
    public String getSuperiorType()
    {
        return (atom_ ? "atom" : "cohesion");
    }
    
    public void setAtom(boolean atom)
    {
        this.atom_ = atom;
    }
    
    public boolean isAtom()
    {
        return atom_;
    }
    
    public boolean isCohesion()
    {
        return!atom_;
    }
    
    public void addSuperiorAddress(Address add)
    {
        if (supaddr_ == null)
        {
            supaddr_ = new Vector();
        }
        supaddr_.addElement(add);
    }
    
    public Vector getSuperiorAddresses()
    {
        return supaddr_;
    }
    
    public int getSuperiorAddressesCount()
    {
        return (supaddr_ == null ? 0 : supaddr_.size());
    }
    
    public Address getSuperiorAddress(int pos)
    {
        if (supaddr_ == null || pos >= supaddr_.size())
        {
            return null;
        }
        else
        {
            return (Address)supaddr_.elementAt(pos);
        }
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:context");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        for (int i = 0; i < supaddr_.size(); i++)
        {
            XElement addr = new XElement("btp:superior-address");
            addr.addElement(((Address)supaddr_.elementAt(i)).toXml());
            el.addElement(addr);
        }
        el.addElement(new XElement("btp:superior-identifier", supid_));
        el.addElement(
            new XElement("btp:superior-type", (atom_ ? "atom" : "cohesion")));
        addQualifiers(el);
        addReplyAddress(el);
        return el;
    }
     public Metaset toServiceSet(){
    	
    	Metaset   con_s=new Metaset("btp:context");
    	if(id_!=null){
    	  con_s.add(new Attribute(new QName("id"),id_));
    	}
    	    	 for (int i = 0; i < supaddr_.size(); i++){
    	 	Metaset addr_s=new Metaset("btp:superior-address");
    	 	addr_s.add(((Address)supaddr_.elementAt(i)).toServiceSet());
    	 	con_s.add(addr_s);
    	 }
    	     	 Metaset superiorid_s=new Metaset(new QName("btp:superior-identifier"));
    	 superiorid_s.setValue(supid_);
    	 con_s.add(superiorid_s);
    	     	 Metaset superiortype_s=new Metaset(new QName("btp:superior-type"));
    	 superiortype_s.setValue((atom_ ? "atom" : "cohesion"));
    	 con_s.add(superiortype_s);
    	      	              	 
    	 return con_s;
    }
    
   
    public static void main(String args[])
    {
        Context ctx = new Context("sup-addr-202.112.137.7", "supid1");
        ctx.setSuperiorType("cohesion");
        System.out.println(ctx.toXml().toString());
        System.out.println("Դӡ");
        ctx = new Context(ctx.toXml());
        System.out.println(ctx.toXml().toString());
        Metaset st=ctx.toServiceSet();
        System.out.print(st.filter(".\\btp:superior-address").toXML().toString());
    }
}
