
package org.act.services.transaction.messages;
import org.act.services.transaction.util.*;


public class Fault extends Message{
   
	 
    private String superiorIdentifier_ = null;
    
    private String inferiorIdentifier_ = null;
    
    private String faultType_ = "general";
    
    private String faultText_ = null;
    
    private String faultData_ = null;
        public static String FAULT_CommunicationFailure = "CommunicationFailure";
    public static String FAULT_DuplicateInferior = "DuplicateInferior";
    public static String FAULT_General = "General";
    public static String FAULT_InvalidDecider = "InvalidDecider";
    public static String FAULT_InvalidInferior = "InvalidInferior";
    public static String FAULT_InvalidSuperior = "InvalidSuperior";
    public static String FAULT_StatusRefused = "StatusRefused";
    public static String FAULT_InvalidTerminator = "InvalidTerminator";
    public static String FAULT_UnknownParameter = "UnknownParameter";
    public static String FAULT_UnkownTransaction = "UnkownTransaction";
    public static String FAULT_UnsupportedQualifier = "UnsupportedQualifier";
    public static String FAULT_WrongState = "WringState";
    public static String FAULT_Redirect = "Redirect";
    
    public Fault()
    {
        super("btp:fault", Message.TYPE_FAULT);
    }
    
    public Fault(String ftype, String fdata, String ftext)
    {
        super("btp:fault", Message.TYPE_FAULT);
        if (ftype != null)
        {
            faultType_ = ftype;
        }
        faultText_ = ftext;
        faultData_ = fdata;
    }
    
    public Fault(XElement el)
    {
        super(el, Message.TYPE_FAULT);
        XHashtable h = el.toHashtable();
        id_ = el.getAttribute("id");
        Object obj = h.getSingle("/btp:fault/btp:superior-identifier");
        if (obj != null)
        {
            superiorIdentifier_ = obj.toString();
        }
        obj = h.getSingle("/btp:fault/btp:fault-type");
        if (obj != null)
        {
            faultType_ = obj.toString();
        }
        obj = h.getSingle("/btp:fault/btp:fault-text");
        if (obj != null)
        {
            faultText_ = obj.toString();
        }
        obj = h.getSingle("/btp:fault/btp:fault-data");
        if (obj != null)
        {
            faultData_ = obj.toString();
        }
    }
    
    public void setSuperiorIdentifier(String uri)
    {
        superiorIdentifier_ = uri;
    }
    
    public String getSuperiorIdentifier()
    {
        return superiorIdentifier_;
    }
    
    public void setInferiorIdentifier(String uri)
    {
        inferiorIdentifier_ = uri;
    }
    
    public String getInferiorIdentifier()
    {
        return inferiorIdentifier_;
    }
    
    public void setFaultText(String text)
    {
        faultText_ = text;
    }
    
    public String getFaultText()
    {
        return faultText_;
    }
    
    public void setFaultType(String type)
    {
        faultType_ = type;
    }
    
    public String getFaultType()
    {
        return faultType_;
    }
    
    public void setFaultData(String data)
    {
        faultData_ = data;
    }
    
    public String getFaultData()
    {
        return faultData_;
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:fault");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        el.addElement(
            new XElement("btp:superior-identifier", superiorIdentifier_));
        el.addElement(
            new XElement("btp:inferior-identifier", inferiorIdentifier_));
        el.addElement(new XElement("btp:fault-type", faultType_));
        if (faultData_ != null)
        {
            el.addElement(new XElement("btp:fault-data", faultData_));
        }
        if (faultText_ != null)
        {
            el.addElement(new XElement("btp:fault-text", faultText_));
        }
        addQualifiers(el);
        addTargetAddress(el);
        return el;
    }
    public static void main(String args[])
    {
        System.out.println("===== Build fault");
        Fault obj = new Fault();
        obj.setSuperiorIdentifier("http://www.myaddress.com:8080/superiorid");
        obj.setInferiorIdentifier("http://www.myaddress.com:8080/inferiorid");
        obj.setFaultData("fault data");
        obj.setFaultText("...string data...");
        System.out.println(obj.toString());
        System.out.println("===== Extract fault");
        obj = new Fault(obj.toXml());
        System.out.println(obj.toString());
    }
}
