
package org.act.services.transaction.messages;
import org.act.services.transaction.util.*;
import java.util.*;


public class InferiorStatuses extends Message{
    
    private String respid_;
    
    private Vector statitems_ = null;
    
    public InferiorStatuses(String responseId)
    {
        super("btp:inferior-statuses", Message.TYPE_INFERIOR_STATUSES);
        respid_ = responseId;
        statitems_ = new Vector();
    }
    
    public InferiorStatuses(XElement el)
    {
        super(el, Message.TYPE_INFERIOR_STATUSES);
        String ns = el.getNamespace();
        ns = (ns == null ? "" : ns + ":");
        XHashtable h = el.toHashtable();
        id_ = el.getAttribute("id");
        respid_ = pickString(h,
                             "/" + name_ + "/" + ns + "responders-identifier");
        Vector v = h.get("/" + name_ + "/" + ns + "status-list");
        if (v != null)
        {
            for (int i = 0; i < v.size(); i++)
            {
                addStatusItem(new StatusItem((XElement)v.elementAt(i)));
            }
        }
    }
    
    public String getRespondersIdentifier()
    {
        return respid_;
    }
    
    public void addStatusItem(StatusItem st)
    {
        if (statitems_ == null)
        {
            statitems_ = new Vector();
        }
        statitems_.addElement(st);
    }
    
    public void addStatusItem(String infid, String status)
    {
        addStatusItem(new StatusItem(infid, status));
    }
    
    public Vector getStatusItems()
    {
        return statitems_;
    }
    
    public StatusItem getStatusItem(int pos)
    {
        if (statitems_ == null || pos >= statitems_.size())
        {
            return null;
        }
        return
            (StatusItem)statitems_.elementAt(pos);
    }
    
    public XElement toXml()
    {
        XElement el = new XElement("btp:inferior-statuses");
        if (id_ != null)
        {
            el.addAttribute("id", id_);
        }
        el.addElement(
            new XElement("btp:responders-identifier", respid_));
        if (statitems_ != null)
        {
            XElement a = new XElement("btp:status-list");
            for (int i = 0; i < statitems_.size(); i++)
            {
                a.addElement(((StatusItem)statitems_.elementAt(i)).toXml());
            }
            el.addElement(a);
        }
        addQualifiers(el);
        addTargetAddress(el);
        return el;
    }
    public static void main(String args[])
    {
        System.out.println("===== Build InferiorStatuses");
        InferiorStatuses obj = new InferiorStatuses("inf1");
        obj.addStatusItem("inf1", "active");
        obj.addStatusItem("inf2", "confirmed");
        obj.addStatusItem("inf3", "prepared");
        System.out.println(obj.toXml().toString());
        System.out.println("===== Extract InferiorStatuses");
        obj = new InferiorStatuses(obj.toXml());
        System.out.println(obj.toString());
    }
	
}
