
package org.act.services.workflow.activity;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Stack;
import java.util.logging.Logger;
import org.act.metaset.Metaset;
import org.act.services.locale.StringManager;
import org.act.services.workflow.context.WorkflowRuntimeContext;
import org.act.services.workflow.debug.Debug;
import org.act.services.workflow.debug.IDebugEngineListener;
import org.act.services.workflow.record.WorkflowActivityRecord;

public class ActivityImpl {
	
	protected static Logger log = Logger.getLogger(ActivityImpl.class.getName());
	protected static StringManager sm =
		StringManager.getManager("org.act.services.locale");
	
	protected Metaset activity;
	protected WorkflowRuntimeContext rc;
	protected Stack stack;
	protected WorkflowActivityRecord war;
	
	public ActivityImpl(Metaset activity, WorkflowRuntimeContext rc, Stack stack) {
		this.activity = activity;
		this.rc = rc;
		this.stack = stack;
		this.war = new WorkflowActivityRecord();
	}
	
	public void process() {
		System.out.println("");
		System.out.println("***"+activity.getName()+": "+ activity.selector(".\\", "name")+"***");
		if (Debug.isDebug()) {
			String debugId = this.activity.selector(".\\", "debugId");
			String breakPoint = this.activity.selector(".\\", "breakPoint");
			if (debugId != null && breakPoint != null) {
				if (breakPoint.equals("true") || Debug.inIdList(debugId)) {
					synchronized (Debug.getRc()) {
						try {
							List listeners = Debug.getListeners();
							for (int i = 0; i < listeners.size(); i++) {
								((IDebugEngineListener) listeners.get(i))
										.stopped(debugId);
							}
							Debug.getRc().wait();
						} catch (InterruptedException e) {
														e.printStackTrace();
						}
					}
				}
			}
		}
	}
	
	public void startProcess() {
		String name = this.activity.selector(".\\", "name");
		String type = this.activity.getName();
		String status = "running";
		String beginTime = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS"))
				.format(new GregorianCalendar().getTime());
		this.war.setName(name);
		this.war.setType(type);
		this.war.setStatus(status);
		this.war.setBeginTime(beginTime);
		this.rc.getActivityRecords().add(this.war);
	}
	
	public void endProcess() {
		String status = "completed";
		String finishTime = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS"))
				.format(new GregorianCalendar().getTime());
		this.war.setStatus(status);
		this.war.setFinishTime(finishTime);
	}
	
	public Metaset getActivity() {
		return activity;
	}
	
	public void setActivity(Metaset activity) {
		this.activity = activity;
	}
	
	public WorkflowRuntimeContext getRc() {
		return rc;
	}
	
	public void setRc(WorkflowRuntimeContext rc) {
		this.rc = rc;
	}
	
	public Stack getStack() {
		return stack;
	}
	
	public void setStack(Stack stack) {
		this.stack = stack;
	}
	
	public WorkflowActivityRecord getWar() {
		return war;
	}
	
	public void setWar(WorkflowActivityRecord war) {
		this.war = war;
	}
}
