
package org.act.services.workflow.context;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Stack;
import org.act.metaset.Metaset;
import org.act.services.transaction.messages.Context;
import org.act.services.workflow.data.CorrelationSet;
import org.act.services.workflow.data.Variable;
import org.act.services.workflow.record.WorkflowInstanceRecord;
import org.act.services.workflow.transaction.TransactionAgent;

public class WorkflowRuntimeContext {
	private WorkflowContext wc;
	
	private String instanceID;
	private String scopeName;
	
	private Variable variables;
	private CorrelationSet correlationSets;
	private Metaset faultHandlers;
	private Metaset compensationHandler;
	private Metaset eventHandlers;
	private Stack activitiesStack = new Stack();
	
	private List activityRecords = new ArrayList(); 
	
	private WorkflowInstanceRecord wir = new WorkflowInstanceRecord();
		private HashMap partnerLinkMgCxtMap = new HashMap();
	
		TransactionAgent agent=new TransactionAgent(); 
		
	private Stack localKeepStack = new Stack();
	public WorkflowRuntimeContext(WorkflowContext wc)
			throws CloneNotSupportedException {
		this.wc = wc;
		this.scopeName = null;
		this.instanceID = null;
		
		this.setVariables(new Variable(wc.getVariables(),this));
		this.setCorrelationSets(new CorrelationSet(wc.getCorrelationSets(),this));
		
		this.setFaultHandlers(wc.getFaultHandlers());
		this.setCompensationHandler(wc.getCompensationHandler());
		this.setEventHandlers(wc.getEventHandlers());
		Metaset temp = (Metaset) wc.getActivity().clone();
		this.activitiesStack.push(temp);	
	}
	
	public void saveLocale() {
	
		localKeepStack.push(eventHandlers);
		localKeepStack.push(compensationHandler);
		localKeepStack.push(faultHandlers);
		localKeepStack.push(correlationSets);
		localKeepStack.push(variables);
		localKeepStack.push(scopeName);
		scopeName = new String();
		variables = new Variable(null,this);
		correlationSets = new CorrelationSet(null,this);
		faultHandlers = new Metaset();
		compensationHandler = new Metaset();
		eventHandlers = new Metaset();
	}
	
	public void resumeLocale() {
		scopeName = (String) this.localKeepStack.pop();
		variables = (Variable) this.localKeepStack.pop();
		correlationSets = (CorrelationSet) this.localKeepStack.pop();
		faultHandlers = (Metaset) this.localKeepStack.pop();
		compensationHandler = (Metaset) this.localKeepStack.pop();
		eventHandlers = (Metaset) this.localKeepStack.pop();
	}
	
	public Stack getActivitiesStack() {
		return activitiesStack;
	}
	
	public void setActivitiesStack(Stack activitiesStack) {
		this.activitiesStack = activitiesStack;
	}
	
	public Metaset getCompensationHandler() {
		return compensationHandler;
	}
	
	public void setCompensationHandler(Metaset compensationHandler) {
		this.compensationHandler = compensationHandler;
	}
	
	public CorrelationSet getCorrelationSets() {
		return correlationSets;
	}
	
	public void setCorrelationSets(CorrelationSet correlationSets) {
		this.correlationSets = correlationSets;
	}
	
	public Metaset getEventHandlers() {
		return eventHandlers;
	}
	
	public void setEventHandlers(Metaset eventHandlers) {
		this.eventHandlers = eventHandlers;
	}
	
	public Metaset getFaultHandlers() {
		return faultHandlers;
	}
	
	public void setFaultHandlers(Metaset faultHandlers) {
		this.faultHandlers = faultHandlers;
	}
	
	public Stack getLocalKeepStack() {
		return localKeepStack;
	}
	
	public void setLocalKeepStack(Stack localKeepStack) {
		this.localKeepStack = localKeepStack;
	}
	
	public HashMap getPartnerLinkMgCxtMap() {
		return partnerLinkMgCxtMap;
	}
	
	public void setPartnerLinkMgCxtMap(HashMap partnerLinkMgCxtMap) {
		this.partnerLinkMgCxtMap = partnerLinkMgCxtMap;
	}
	
	public String getScopeName() {
		return scopeName;
	}
	
	public void setScopeName(String scopeName) {
		this.scopeName = scopeName;
	}
	
	public Variable getVariables() {
		return variables;
	}
	
	public void setVariables(Variable variables) {
		this.variables = variables;
	}
	
	public WorkflowContext getWc() {
		return wc;
	}
	
	public void setWc(WorkflowContext wc) {
		this.wc = wc;
	}
	
	public TransactionAgent getAgent() {
		return agent;
	}
	
	public void setAgent(TransactionAgent agent) {
		this.agent = agent;
	}
	
	public String getInstanceID() {
		return instanceID;
	}
	
	public void setInstanceID(String instanceID) {
		this.instanceID = instanceID;
	}
	
	public List getActivityRecords() {
		return activityRecords;
	}
	
	public void setActivityRecords(List activityRecords) {
		this.activityRecords = activityRecords;
	}
	
	public WorkflowInstanceRecord getWir() {
		return wir;
	}
	
	public void setWir(WorkflowInstanceRecord wir) {
		this.wir = wir;
	}
	
	public Context getTxContext(String transactionName){
		return this.agent.getTxContxt(transactionName);
	}
}
