
package org.act.services.workflow.data;
import java.util.Stack;
import java.util.logging.Logger;
import org.act.metaset.QName;
import org.act.metaset.Metaset;
import org.act.services.locale.StringManager;
import org.act.services.workflow.context.WorkflowRuntimeContext;

public class Variable {
	
	private static Logger log = Logger.getLogger(Variable.class.getName());
	
	private static StringManager sm =
		StringManager.getManager("org.act.services.locale");
	private Metaset variables;
	private WorkflowRuntimeContext rc;
	
	public Variable(Metaset variables, WorkflowRuntimeContext rc) {
		this.rc = rc;
		this.setVars(variables);
	}
	
					public Metaset getVar(String varName) {
		Metaset var = null;
		if (this.variables != null) {
			var = this.variables.filter("variables\\variable", "name", varName);
		}
		if (var != null) {
			return var.car();
		} else {
			Stack localKeepStack = this.rc.getLocalKeepStack();
			for (int i = localKeepStack.size() - 2; i > 0; i = i - 6) {
				Variable variable = (Variable) localKeepStack.get(i);
				Metaset tempVars = variable.getVariables();
				if (tempVars != null) {
					Metaset tempVar = tempVars.filter("variables\\variable",
							"name", varName);
					if (tempVar != null) {
						return tempVar.car();
					}
				}
			}
		}
		
		log.severe(sm.getString("org.act.wswf.error.noSuchVariable",varName));
		
		return null;
	}
	
	public String getVarPartValue(String varName, String partName) {
		Metaset var = this.getVar(varName);
		Metaset parts = var.filter(".\\part", "name", partName);
		if(parts!=null) {
			return parts.car().getValue();
		}
		else{
			log.severe(sm.getString("org.act.wswf.error.noSuchPart",varName,partName));
			return null;
		}
	}
	
	public String getVarPartValue(String varName, int partNo) {
		Metaset var = this.getVar(varName);
		Metaset part = (Metaset) var.getChilds().get(partNo);
		return part.getValue();
	}
	
	public void setVarPartValue(String varName, String partName, String value) {
		Metaset var = this.getVar(varName);
		Metaset part = var.filter(".\\part", "name", partName).car();
		part.setValue(value);
	}
	
	public void setVarPartValue(String varName, int partNo, String value) {
		Metaset var = this.getVar(varName);
		Metaset part = (Metaset) var.getChilds().get(partNo);
		part.setValue(value);
	}
	
	public WorkflowRuntimeContext getRc() {
		return rc;
	}
	
	public void setRc(WorkflowRuntimeContext rc) {
		this.rc = rc;
	}
	
	public Metaset getVariables() {
		return variables;
	}
	
	public void setVariables(Metaset variables) {
		this.variables = variables;
	}
	
					public void setVars(Metaset variables) {
		if (variables != null) {
			try {
				this.variables = (Metaset) variables.clone();
			} catch (CloneNotSupportedException e) {
								e.printStackTrace();
			}
			for (int i = 0; i < this.variables.getChilds().size(); i++) {
				Metaset var = (Metaset) this.variables.getChilds().get(i);
				QName messageName = new QName(var
						.selector(".\\", "messageType"));
				Metaset wsdl = rc.getWc().getWsdlOnNamespace(
						messageName.getNameSpace());
				Metaset messages = wsdl.filter(".\\message");
				Metaset message = messages.filter(".\\*", "name", messageName
						.getName());
				if (message.getChilds().size() != 0) {
					Metaset msg = (Metaset) message.getChilds().get(0);
					Metaset parts = null;
					try {
						parts = (Metaset) msg.filter(".\\part").clone();
					} catch (CloneNotSupportedException e1) {
												e1.printStackTrace();
					}
					for (int j = 0; j < parts.getChilds().size(); j++) {
						Metaset part = (Metaset) parts.getChilds().get(j);
						part.setValue("null");
						var.add(part);
					}
				}
			}
		}
	}
}
