
package org.act.services.workflow.manager;
import java.util.logging.Logger;
import org.act.metaset.Metaset;
import org.act.services.IMessageContext;
import org.act.services.locale.StringManager;
import org.act.services.pool.IObjectPool;
import org.act.services.pool.ObjectPool;
import org.act.services.runtime.Application;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.Terminatable;
import org.act.services.thread.ThreadPool;
import org.act.services.workflow.transaction.WorkflowTransactionManager;

public class WorkflowGlobalManager extends Terminatable {
	private static Logger log = Logger.getLogger(WorkflowGlobalManager.class
			.getName());
	private static StringManager sm = StringManager
			.getManager("org.act.services.locale");
	private ThreadPool threadPool;
		private IObjectPool instancesMessagePool = null;
		private IObjectPool transactionMessagePool = null;
		private IObjectPool manageMessagePool = null;
		private WorkflowInstanceManager wim = null;
		private WorkflowTransactionManager wtm = null;
		private WorkflowEngineManager wem = null;
	
	private Thread instancesThread;
	
	private Thread transactionThread;
	
	private Thread manageThread;
	
	public WorkflowGlobalManager(IObjectPool msgInputPool,
			ThreadPool threadPool, IRuntime runtime) {
		this.msgIn = msgInputPool;
		this.threadPool = threadPool;
		this.runtime = runtime;
				this.instancesMessagePool = new ObjectPool();
				this.transactionMessagePool = new ObjectPool();
				this.manageMessagePool = new ObjectPool();
				wim = new WorkflowInstanceManager(this.instancesMessagePool,
				this.threadPool, this.runtime);
				instancesThread = new Thread(wim);
		instancesThread.setName("Instance Manager Thread");
		instancesThread.start();
				wtm = new WorkflowTransactionManager(this.transactionMessagePool,
				this.msgIn, this.threadPool, this.runtime);
		
				transactionThread = new Thread(wtm);
		transactionThread.start();
				wem = new WorkflowEngineManager(manageMessagePool, this.msgIn,
				this.runtime);
				manageThread = new Thread(wem);
		manageThread.setName("Workflow Manager Thread");
		manageThread.start();
	}
	
	public void stop() {
					wim.terminate();
			synchronized (instancesMessagePool) {
				instancesMessagePool.notifyAll();
			}
								wtm.terminate();
			synchronized (transactionMessagePool) {
				transactionMessagePool.notifyAll();
			}
			
			wem.terminate();
			synchronized (manageMessagePool) {
				manageMessagePool.notifyAll();
			}
	}
	private IObjectPool msgIn = null;
	private IRuntime runtime = null;
	
	protected void process() throws ObjectOperationException {
								IMessageContext mc = null;
		synchronized (msgIn) {
			while (msgIn.empty()) {
				switch (getState()) {
				case RUNNING:
					try {
						msgIn.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					break;
				case STOPPING:
					return;
				default:
					throw new IllegalMonitorStateException();
				}
			}
			mc = (IMessageContext) msgIn.get();
			msgIn.notifyAll();
		}
				Metaset requestService = mc.getRequestServices();
		if (Application.isDebug()) {
			System.out.println("WorkflowGlobalManager.process():");
			System.out.println(requestService.toXML());
		}
				Metaset transactionContext = requestService
				.filter(".\\Header\\TransactionID");
				Metaset manageContext = requestService.filter(".\\Header\\ManageID");
		if (transactionContext != null) {
			if (Application.isDebug()) {
				System.out.println("transactionContext");
			}
			synchronized (this.transactionMessagePool) {
				this.transactionMessagePool.put(mc);
				this.transactionMessagePool.notifyAll();
			}
		} else if (manageContext != null) {
			if (Application.isDebug()) {
				System.out.println("manageContext");
			}
			synchronized (this.manageMessagePool) {
				this.manageMessagePool.put(mc);
				this.manageMessagePool.notifyAll();
			}
		}
		else {
			if (Application.isDebug()) {
				System.out.println("instancesContext");
			}
			String serviceName = mc.getServicesName();
			String operation = requestService.filter(".\\Body\\*").car()
					.getName();
			log.info(sm.getString("org.act.wswf.recMessage", serviceName,
					operation));
			synchronized (this.instancesMessagePool) {
				this.instancesMessagePool.put(mc);
				this.instancesMessagePool.notifyAll();
			}
		}
	}
}
