
package org.act.services.workflow.manager;
import org.act.metaset.Metaset;
import org.act.services.IMessageContext;
import org.act.services.MessageContext;
import org.act.services.locale.StringManager;
import org.act.services.pool.IObjectPool;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.Terminatable;
import org.act.services.thread.ThreadPool;
import org.act.services.thread.ThreadPoolRunnable;
import org.act.services.workflow.context.IWorkflowContextPool;
import org.act.services.workflow.context.IWorkflowRuntimeContextPool;
import org.act.services.workflow.context.WorkflowContext;
import org.act.services.workflow.context.WorkflowInstance;
import org.act.services.workflow.context.WorkflowRuntimeContext;
import org.act.services.workflow.debug.Debug;
import org.act.services.workflow.debug.IDebugEngineListener;
import org.act.services.workflow.record.Records2DB;
import java.net.InetAddress;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.logging.Logger;

public class WorkflowInstanceManager extends Terminatable {
	private int count = 0;
	private ThreadPool threadPool;
	static Logger log = Logger.getLogger(WorkflowInstanceManager.class
			.getName());
	static StringManager sm = StringManager
			.getManager("org.act.services.locale");
	
	public WorkflowInstanceManager(IObjectPool msgInputPool,
			ThreadPool threadPool, IRuntime runtime) {
		this.msgIn = msgInputPool;
		this.threadPool = threadPool;
		this.runtime = runtime;
	}
	private IObjectPool msgIn = null;
	private IRuntime runtime = null;
	
	protected void process() throws ObjectOperationException {
		synchronized (msgIn) {
			for (int i = 0; i < msgIn.size(); i++) {
				final IMessageContext context = (IMessageContext) msgIn.get(i);
				final WorkflowContext wc = IWorkflowContextPool.INSTANCE
						.getWorkflowContext(context.getServicesName());
				if (((context.getStatusCode() & MessageContext.CAN_NOT_CREATE_INSTANCE) == 0)) {
					if (wc != null && wc.isDeploy()) {
						if (this.isEntryPoint(wc, context.getRequestServices())) {
							final int num = inc();
							log.info(sm.getString(
									"org.act.wswf.createInstance", context
											.getServicesName()
											+ " (" + num + ")"));
							ThreadPoolRunnable runnable = new ThreadPoolRunnable() {
								public Object[] getInitData() {
									return null;
								}
								public void runIt(Object[] thData) {
									WorkflowRuntimeContext rc;
									try {
										rc = new WorkflowRuntimeContext(wc);
										IWorkflowRuntimeContextPool.INSTANCE
												.add(rc);
																				String instanceID = createInstanceId();
																				rc.setInstanceID(instanceID);
									} catch (CloneNotSupportedException e) {
																				e.printStackTrace();
										rc = null;
									}
									WorkflowInstance workflow = new WorkflowInstance(
											rc, msgIn, runtime);
									log.info(sm.getString(
											"org.act.wswf.beginInstance",
											context.getServicesName() + " ("
													+ num + ")"));
									workflow.run();
									Records2DB.instance2DB(rc);
									if (Debug.isDebug()) {
										Debug.setRc(null);
										List listeners = Debug.getListeners();
										for (int i = 0; i < listeners.size(); i++) {
											((IDebugEngineListener) listeners
													.get(i)).stopped(null);
										}
									}
									IWorkflowRuntimeContextPool.INSTANCE
											.remove(rc);
									log.info(sm.getString(
											"org.act.wswf.endInstance", context
													.getServicesName()
													+ " (" + num + ")"));
								}
							};
							threadPool.runIt(runnable);
						}
					} else {
																							}
				} else {
										context.setStatusCode(context.getStatusCode()
							| MessageContext.CAN_NOT_CREATE_INSTANCE);
				}
			}
			try {
				msgIn.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	private boolean isEntryPoint(WorkflowContext wc, Metaset requestService) {
		boolean createInstance = false;
				String operationName = requestService.selector(".\\Body\\*");
		List entryList = wc.getEntryList();
		for (int i = 0; i < entryList.size(); i++) {
			Metaset entryItem = (Metaset) entryList.get(i);
			String entryTag = entryItem.getName();
			String entryItemOp = null;
			if (entryTag.equals("receive")) {
				entryItemOp = entryItem.selector(".\\", "operation");
				if (entryItemOp.equals(operationName)) {
					createInstance = true;
					break;
				}
			} else if (entryTag.equals("pick")) {
				Metaset onMessages = entryItem.filter(".\\onMessage");
				for (int j = 0; j < onMessages.getChilds().size(); j++) {
					Metaset onMessage = (Metaset) onMessages.getChilds().get(j);
					entryItemOp = onMessage.selector(".\\", "operation");
					if (entryItemOp.equals(operationName)) {
						createInstance = true;
						break;
					}
				}
			}
			if (createInstance == true)
				break;
		}
		return createInstance;
	}
	
	synchronized public static String createInstanceId() {
		String NameOfHost = null;
		String id = null;
		try {
			InetAddress localAddress = InetAddress.getLocalHost();
			NameOfHost = localAddress.getHostAddress();
			Date date = new Date();
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
			String time = sdf.format(date).toString();
			id = time + "-" + NameOfHost + "-" + System.currentTimeMillis() + "-" + Math.abs((new Random()).nextLong());
			return id;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return id;
	}
	synchronized public int inc() {
		count++;
		return count;
	}
}
